@@ -1,13 +1,11 @@
 # created by Dmitrey
-#PythonAll = all
+
 from numpy import asarray, empty, inf, any, array, \
 asfarray, isscalar, ndarray, int16, int32, int64, float64, tile, vstack, searchsorted, logical_or, where, \
-asanyarray, arange, log2, logical_and, ceil, string_, atleast_1d
+asanyarray, arange, log2, logical_and, ceil
 import numpy as np
 from FDmisc import FuncDesignerException, isPyPy
-from ooFun import oofun
-from logic import AND, OR, NOT, EQUIVALENT
-from BooleanOOFun import BooleanOOFun
+from ooFun import oofun, BooleanOOFun, AND, OR, NOT, EQUIVALENT
 #from FuncDesigner import IMPLICATION
 from ooarray import ooarray
 #from FuncDesigner.Interval import adjust_lx_WithDiscreteDomain, adjust_ux_WithDiscreteDomain
@@ -17,7 +15,7 @@ from boundsurf import boundsurf, surf
 
 f_none = lambda *args, **kw: None
 One = array(1.0)
-Zero = array(0.0) ## TODO: mb use 1-d array to improve indexation
+Zero = array(0.0)
 class oovar(oofun):
     is_oovar = True
     domain = None
@@ -38,31 +36,20 @@ class oovar(oofun):
             kwargs['name'] = name
         oofun.__init__(self, f_none, *args, **kwargs)
     
-    def expression(self, *args, **kw):
-        return self.name
-    
     def _interval_(self, domain, dtype = float64):
-        if 0 or self in domain.resolveSchedule:
+        if self in domain.resolveSchedule:
             tmp = domain.get(self, None)
             if tmp is None: return None
             if isinstance(tmp, ndarray) or isscalar(tmp): # thus variable value is fixed for this calculation
                 tmp = asarray(tmp, dtype)
                 return tile(tmp, (2, 1)), True
             infinum, supremum = tmp
-            
-            #prev
-            #return asarray(vstack((infinum, supremum)), dtype), True
-            #new, works faster in CPython
-            r = empty((2, asarray(infinum).size), dtype)
-            r[0] = infinum
-            r[1] = supremum
-            return r, True
+            return asarray(vstack((infinum, supremum)), dtype), True
         else:
             S = surf({self: One}, Zero)
             return boundsurf(S, S, True, domain), True
     
     def _getFuncCalcEngine(self, x, **kwargs):
-        #print x
         if hasattr(x, 'xf'):
             #return x.xf[self]
             if x.probType == 'MOP':
@@ -135,34 +122,19 @@ class oovar(oofun):
         else:
             return oofun.__eq__(self, other)
     
-    def formAuxDomain(self, sort = True):
+    def formAuxDomain(self):
         if 'aux_domain' in self.__dict__: return
+        self.domain = asanyarray(self.domain)
         d = self.domain
 #        if d.dtype.type not in [string_, unicode, str]:
 #            raise FuncDesignerException('to compare string with oovar latter should have domain of string type')
-        if type(d[0]) in (str, string_):
-            d = dict((elem, i) for i, elem in enumerate(d))
-            D = int(2 ** ceil(log2(len(d))))
-            self.reverse_aux_domain = dict((i, elem) for i, elem in enumerate(self.domain))
-        elif sort:    
-            d = asanyarray(d)
-            if any(d[1:] > d[:-1]):
-#                if type(d) == tuple:
-#                    d = list(d)
-                d.sort()
-            #self.ub = d.size - 1
-            
-            D = int(2 ** ceil(log2(len(atleast_1d(d)))))
-        else:
-            d = asanyarray(d)
-        # atleast_1d - for domain from 1 element if it will be somewhere generated and obtained here
-        
-        self.domain, self.aux_domain = arange(D), d    
-#        self.domainSortOrder = \
-#        1 if PythonAll(d[i] <= d[i+1] for i in range(D-1)) else \
-#        -1 if PythonAll(d[i] >= d[i+1] for i in range(D-1)) else\
-#        0
 
+        if any(d[1:] < d[:-1]):
+            d.sort()
+        #self.ub = d.size - 1
+        D = int(2 ** ceil(log2(d.size)))
+        self.domain, self.aux_domain = arange(D), d    
+    
 #        if isinstance(x, dict):
 #            tmp = x.get(self, None)
 #            if tmp is not None:
