@@ -1,11 +1,9 @@
 PythonSum = sum
 PythonMax = max
 PythonAny = any
-PythonAll = all
 import numpy as np
-from FDmisc import FuncDesignerException, Diag, Eye, raise_except, \
-diagonal, DiagonalType, dictSum, Vstack, Copy
-from ooFun import atleast_oofun, oofun
+from FDmisc import FuncDesignerException, Diag, Eye, raise_except, diagonal, DiagonalType, dictSum
+from ooFun import atleast_oofun, Vstack, Copy, oofun
 from ooarray import ooarray
 from Interval import nonnegative_interval, ZeroCriticalPointsInterval, \
 box_1_interval, defaultIntervalEngine
@@ -156,7 +154,7 @@ def sin(inp):
     elif hasStochastic and isinstance(inp, distribution.stochasticDistribution):
         return distribution.stochasticDistribution(sin(inp.values), inp.probabilities.copy())._update(inp)
     elif not isinstance(inp, oofun): return np.sin(inp)
-    r = oofun(st_sin, inp, engine = 'sin', 
+    r = oofun(st_sin, inp, 
                  d = lambda x: Diag(np.cos(x)), 
                  vectorized = True)
     r._interval_ = lambda domain, dtype: sin_cos_interval(r, inp, domain, dtype)
@@ -176,7 +174,7 @@ def cos(inp):
     elif hasStochastic and isinstance(inp, distribution.stochasticDistribution):
         return distribution.stochasticDistribution(cos(inp.values), inp.probabilities.copy())._update(inp)
     elif not isinstance(inp, oofun): return np.cos(inp)
-    r = oofun(st_cos, inp, engine = 'cos', 
+    r = oofun(st_cos, inp, 
                  d = lambda x: Diag(-np.sin(x)), 
                  vectorized = True)
     r._interval_ = lambda domain, dtype: sin_cos_interval(r, inp, domain, dtype)
@@ -207,7 +205,7 @@ def tan(inp):
     if not isinstance(inp, oofun): return np.tan(inp)
     # TODO: move it outside of tan definition
     r = oofun(st_tan, inp, d = lambda x: Diag(1.0 / np.cos(x) ** 2), vectorized = True, \
-    engine_monotonity = 1, convexities = (-1, 1), engine = 'tan')
+    engine_monotonity = 1, convexities = (-1, 1))
     r._interval_ = lambda domain, dtype: tan_interval(inp, r, domain, dtype)
     return r
     
@@ -256,7 +254,7 @@ def arccos(inp):
         return distribution.stochasticDistribution(arccos(inp.values), inp.probabilities.copy())._update(inp)     
     if not isinstance(inp, oofun): return np.arccos(inp)
     r = oofun(st_arccos, inp, d = lambda x: Diag(-1.0 / np.sqrt(1.0 - x**2)), vectorized = True, 
-    engine_monotonity = -1, convexities = (1, -1), engine = 'arccos')
+    engine_monotonity = -1, convexities = (1, -1))
 #    r.getDefiniteRange = get_box1_DefiniteRange
     r._interval_ = lambda domain, dtype: box_1_interval(inp, r, np.arccos, domain, dtype)
     r.attach((inp>-1)('arccos_domain_lower_bound_%d' % r._id, tol=-1e-7), (inp<1)('arccos_domain_upper_bound_%d' % r._id, tol=-1e-7))
@@ -277,7 +275,7 @@ def arctan(inp):
         return distribution.stochasticDistribution(arctan(inp.values), inp.probabilities.copy())._update(inp)             
     if not isinstance(inp, oofun): return np.arctan(inp)
     r = oofun(st_arctan, inp, d = lambda x: Diag(1.0 / (1.0 + x**2)), 
-                 vectorized = True, engine_monotonity = 1, convexities = (1, -1), engine = 'arctan')
+                 vectorized = True, engine_monotonity = 1, convexities = (1, -1))
     r._interval_ = lambda domain, dtype: devided_interval(inp, r, domain, dtype)
     return r
 
@@ -298,7 +296,7 @@ def sinh(inp):
         return distribution.stochasticDistribution(sinh(inp.values), inp.probabilities.copy())._update(inp)        
     if not isinstance(inp, oofun): return np.sinh(inp)
     r = oofun(st_sinh, inp, d = lambda x: Diag(np.cosh(x)), vectorized = True, 
-    engine_monotonity = 1, convexities = (-1, 1), engine = 'sinh')
+    engine_monotonity = 1, convexities = (-1, 1))
     r._interval_ = lambda domain, dtype: devided_interval(inp, r, domain, dtype)
     return r
 
@@ -341,9 +339,8 @@ def cosh(inp):
         return distribution.stochasticDistribution(cosh(inp.values), inp.probabilities.copy())._update(inp)                
     if not isinstance(inp, oofun): 
         return np.cosh(inp)
-    r =oofun(st_cosh, inp, d = lambda x: Diag(np.sinh(x)), engine_convexity = 1, vectorized = True, \
-    _interval_=ZeroCriticalPointsInterval(inp, np.cosh), engine = 'cosh')
-    return r
+    return oofun(st_cosh, inp, d = lambda x: Diag(np.sinh(x)), engine_convexity = 1, vectorized = True, \
+    _interval_=ZeroCriticalPointsInterval(inp, np.cosh))
     
 __all__ += ['sinh', 'cosh']
 
@@ -362,7 +359,7 @@ def tanh(inp):
         return distribution.stochasticDistribution(tanh(inp.values), inp.probabilities.copy())._update(inp)              
     if not isinstance(inp, oofun): return np.tanh(inp)
     r = oofun(st_tanh, inp, d = lambda x: Diag(1.0/np.cosh(x)**2), vectorized = True, 
-    engine_monotonity = 1, convexities = (1, -1), engine = 'tanh')
+    engine_monotonity = 1, convexities = (1, -1))
     r._interval_ = lambda domain, dtype: devided_interval(inp, r, domain, dtype)
     return r
     
@@ -381,7 +378,7 @@ def arctanh(inp):
         return distribution.stochasticDistribution(arctanh(inp.values), inp.probabilities.copy())._update(inp)          
     if not isinstance(inp, oofun): return np.arctanh(inp)
     r = oofun(st_arctanh, inp, d = lambda x: Diag(1.0/(1.0-x**2)), vectorized = True, 
-    engine_monotonity = 1, convexities = (-1, 1), engine = 'arctanh')
+    engine_monotonity = 1, convexities = (-1, 1))
 #    r.getDefiniteRange = get_box1_DefiniteRange
     r._interval_ = lambda domain, dtype: box_1_interval(inp, r, np.arctanh, domain, dtype)
     return r
@@ -403,7 +400,7 @@ def arcsinh(inp):
         return distribution.stochasticDistribution(arcsinh(inp.values), inp.probabilities.copy())._update(inp)      
     if not isinstance(inp, oofun): return np.arcsinh(inp)
     r = oofun(st_arcsinh, inp, d = lambda x: Diag(1.0/np.sqrt(1+x**2)), 
-    vectorized = True, engine_monotonity = 1, convexities = (1, -1), engine = 'arcsinh')
+    vectorized = True, engine_monotonity = 1, convexities = (1, -1))
     r._interval_ = lambda domain, dtype: devided_interval(inp, r, domain, dtype)
     return r
 
@@ -422,7 +419,7 @@ def arccosh(inp):
         return distribution.stochasticDistribution(arccosh(inp.values), inp.probabilities.copy())._update(inp)      
     if not isinstance(inp, oofun): return np.arccosh(inp)
     r = oofun(st_arccosh, inp, d = lambda x: Diag(1.0/np.sqrt(x**2-1.0)), vectorized = True, 
-    engine_monotonity = 1, engine_convexity = -1, engine = 'arccosh')
+    engine_monotonity = 1, engine_convexity = -1)
     F0, shift = 0.0, 1.0
     r._interval_ = lambda domain, dtype: nonnegative_interval(inp, np.arccosh, r.d, domain, dtype, F0, shift)
     return r
@@ -552,7 +549,7 @@ def exp(inp):
         return distribution.stochasticDistribution(exp(inp.values), inp.probabilities.copy())._update(inp)      
     if not isinstance(inp, oofun): return np.exp(inp)
     r = oofun(st_exp, inp, d = lambda x: Diag(np.exp(x)), vectorized = True, 
-    engine_convexity = 1, engine_monotonity = 1, engine = 'exp')
+    engine_convexity = 1, engine_monotonity = 1)
     r._interval_ = lambda domain, dtype: exp_interval(r, inp, domain, dtype)
     return r
 
@@ -575,7 +572,6 @@ def sqrt(inp, attachConstraints = True):
 #    engine_monotonity = 1, engine_convexity = -1)
 #    r._interval_ = lambda domain, dtype: nonnegative_interval(inp, np.sqrt, r.d, domain, dtype, 0.0)
     r = inp ** 0.5
-    r.engine = 'sqrt'
     if attachConstraints: r.attach((inp>0)('sqrt_domain_zero_bound_%d' % r._id, tol=-1e-7))
     return r
 
@@ -596,10 +592,8 @@ def abs(inp):
         return distribution.stochasticDistribution(abs(inp.values), inp.probabilities.copy())._update(inp)      
     if not isinstance(inp, oofun): return np.abs(inp)
     
-    r = oofun(st_abs, inp, d = lambda x: Diag(np.sign(x)), vectorized = True, 
-    _interval_ = ZeroCriticalPointsInterval(inp, np.abs), engine = 'abs')
-    return r
-    
+    return oofun(st_abs, inp, d = lambda x: Diag(np.sign(x)), vectorized = True, _interval_ = ZeroCriticalPointsInterval(inp, np.abs))
+
 __all__ += ['abs']
 
 def get_log_b2_coeffs(L, U, d_l, d_u, c_l, c_u):
@@ -690,8 +684,7 @@ def log(inp):
         return distribution.stochasticDistribution(log(inp.values), inp.probabilities.copy())._update(inp)      
     if not isinstance(inp, oofun): return np.log(inp)
     d = lambda x: Diag(1.0/x)
-    r = oofun(st_log, inp, d = d, vectorized = True, engine_monotonity = 1, engine_convexity = -1, 
-              engine = 'log')
+    r = oofun(st_log, inp, d = d, vectorized = True, engine_monotonity = 1, engine_convexity = -1)
     #r._interval_ = lambda domain, dtype: nonnegative_interval(inp, np.log, r.d, domain, dtype, 0.0)
     r._interval_ = lambda domain, dtype: log_interval(inp, domain, dtype)
     r.attach((inp>1e-300)('log_domain_zero_bound_%d' % r._id, tol=-1e-7))
@@ -718,8 +711,7 @@ def log10(inp):
         return distribution.stochasticDistribution(log10(inp.values), inp.probabilities.copy())._update(inp)              
     if not isinstance(inp, oofun): return np.log10(inp)
     d = lambda x: Diag(INV_LOG_10 / x)
-    r = oofun(st_log10, inp, d = d, vectorized = True, engine_monotonity = 1, engine_convexity = -1, 
-              engine = 'log10')
+    r = oofun(st_log10, inp, d = d, vectorized = True, engine_monotonity = 1, engine_convexity = -1)
     #r._interval_ = lambda domain, dtype: nonnegative_interval(inp, np.log10, r.d, domain, dtype, 0.0)
     r._interval_ = lambda domain, dtype: log10_interval(inp, domain, dtype)
     r.attach((inp>1e-300)('log10_domain_zero_bound_%d' % r._id, tol=-1e-7))
@@ -746,8 +738,7 @@ def log2(inp):
         return distribution.stochasticDistribution(log2(inp.values), inp.probabilities.copy())._update(inp)       
     if not isinstance(inp, oofun): return np.log2(inp)
     d = lambda x: Diag(INV_LOG_2/x)
-    r = oofun(st_log2, inp, d = d, vectorized = True, engine_monotonity = 1, engine_convexity = -1, 
-              engine = 'log2')
+    r = oofun(st_log2, inp, d = d, vectorized = True, engine_monotonity = 1, engine_convexity = -1)
 #    r._interval_ = lambda domain, dtype: nonnegative_interval(inp, np.log2, r.d, domain, dtype, 0.0)
     r._interval_ = lambda domain, dtype: log2_interval(inp, domain, dtype)
     r.attach((inp>1e-300)('log2_domain_zero_bound_%d' % r._id, tol=-1e-7))
@@ -773,7 +764,7 @@ def d_dot(x, y):
 
 def dot(inp1, inp2):
     if not isinstance(inp1, oofun) and not isinstance(inp2, oofun): return np.dot(inp1, inp2)
-    r = oofun(f_dot, [inp1, inp2], d=(lambda x, y: d_dot(x, y), lambda x, y: d_dot(y, x)), engine = 'dot')
+    r = oofun(f_dot, [inp1, inp2], d=(lambda x, y: d_dot(x, y), lambda x, y: d_dot(y, x)))
     r.getOrder = lambda *args, **kwargs: (inp1.getOrder(*args, **kwargs) if isinstance(inp1, oofun) else 0) + (inp2.getOrder(*args, **kwargs) if isinstance(inp2, oofun) else 0)
     #r.isCostly = True
     return r
@@ -785,10 +776,8 @@ def cross_d(x, y):
 def cross(a, b):
     if not isinstance(a, oofun) and not isinstance(b, oofun): return np.cross(a, b)
    
-    r = oofun(np.cross, [a, b], d=(lambda x, y: -cross_d(x, y), lambda x, y: cross_d(y, x)), engine = 'cross')
-    r.getOrder = lambda *args, **kwargs: \
-    (a.getOrder(*args, **kwargs) if isinstance(a, oofun) else 0)\
-    + (b.getOrder(*args, **kwargs) if isinstance(b, oofun) else 0)
+    r = oofun(np.cross, [a, b], d=(lambda x, y: -cross_d(x, y), lambda x, y: cross_d(y, x)))
+    r.getOrder = lambda *args, **kwargs: (a.getOrder(*args, **kwargs) if isinstance(a, oofun) else 0) + (b.getOrder(*args, **kwargs) if isinstance(b, oofun) else 0)
     return r
 
 __all__ += ['dot', 'cross']
@@ -797,7 +786,7 @@ def ceil(inp):
     if isinstance(inp, ooarray) and any(isinstance(elem, oofun) for elem in atleast_1d(inp)):
         return ooarray([ceil(elem) for elem in inp])        
     if not isinstance(inp, oofun): return np.ceil(inp)
-    r = oofun(lambda x: np.ceil(x), inp, vectorized = True, engine_monotonity = 1, engine = 'ceil')
+    r = oofun(lambda x: np.ceil(x), inp, vectorized = True, engine_monotonity = 1)
     r._D = lambda *args, **kwargs: raise_except('derivative for FD ceil is unimplemented yet')
     return r
 
@@ -805,7 +794,7 @@ def floor(inp):
     if isinstance(inp, ooarray) and any(isinstance(elem, oofun) for elem in atleast_1d(inp)):
         return ooarray([floor(elem) for elem in inp])        
     if not isinstance(inp, oofun): return np.floor(inp)
-    r = oofun(lambda x: np.floor(x), inp, vectorized = True, engine_monotonity = 1, engine = 'floor')
+    r = oofun(lambda x: np.floor(x), inp, vectorized = True, engine_monotonity = 1)
     r._D = lambda *args, **kwargs: raise_except('derivative for FD floor is unimplemented yet')
     return r
 
@@ -824,7 +813,7 @@ def sign(inp):
         return distribution.stochasticDistribution(sign(inp.values), inp.probabilities.copy())._update(inp)      
     if not isinstance(inp, oofun): 
         return np.sign(inp)
-    r = oofun(st_sign, inp, vectorized = True, d = lambda x: 0.0, engine_monotonity = 1, engine = 'sign')
+    r = oofun(st_sign, inp, vectorized = True, d = lambda x: 0.0, engine_monotonity = 1)
     return r
 
 __all__ += ['ceil', 'floor', 'sign']
@@ -871,7 +860,7 @@ def sum_interval(R0, r, INP, domain, dtype):
 #        _r = [R0]
 #        DefiniteRange = True
 #        for inp in INP:
-#            arg_lb_ub, definiteRange = inp._interval(domain, dtype, ia_surf_level = 2)
+#            arg_lb_ub, definiteRange = inp._interval(domain, dtype, ia_surf_level = 1)
 #            DefiniteRange = np.logical_and(DefiniteRange, definiteRange)
 #            if type(arg_lb_ub) in (boundsurf, boundsurf2):
 #                B.append(arg_lb_ub)
@@ -911,13 +900,7 @@ def sum_interval(R0, r, INP, domain, dtype):
             return R, DefiniteRange
 
         #R = R.copy() - domain.storedSums[r][v]
-        if type(R) == np.ndarray:
-            assert type(domain.storedSums[r][v]) == np.ndarray
-            R = (R - domain.storedSums[r][v])# not inplace!
-        else:
-            # boundsurf
-            R = R.direct_sub(domain.storedSums[r][v])
-            
+        R = (R - domain.storedSums[r][v])# not inplace!
 #        if R.__class__ == boundsurf:
 #            R = R.resolve()[0] 
         R2 = np.zeros((2, 1))
@@ -1012,34 +995,24 @@ def sum_interval(R0, r, INP, domain, dtype):
 
     
 def boundsurf_sum(B, s, DefiniteRange, domain):
-    
-    sameBounds = np.array_equal(s[0], s[1]) and PythonAll(b.l is b.u for b in B)
     L = PythonSum(b.l.c for b in B) + s[0]
-    U = L if sameBounds else PythonSum(b.u.c for b in B) + s[1]
+    U = PythonSum(b.u.c for b in B) + s[1]
     
     ##############################
     # dictSum works with Python lists, not iterables!
     Ld = dictSum([b.l.d for b in B])
-    Ud = Ld if sameBounds else dictSum([b.u.d for b in B])
+    Ud = dictSum([b.u.d for b in B])
     ##############################
     
     l2 = [elem for elem in (getattr(b.l, 'd2', None) for b in B) if elem is not None]
-    u2 = l2 if sameBounds else [elem for elem in (getattr(b.u, 'd2', None) for b in B) if elem is not None]
+    u2 = [elem for elem in (getattr(b.u, 'd2', None) for b in B) if elem is not None]
 
     if len(l2) == len(u2) == 0:
-        if sameBounds:
-            s1 = s2 = surf(Ld, L)
-        else:
-            s1, s2 = surf(Ld, L), surf(Ud, U)
-        return boundsurf(s1, s2, DefiniteRange, domain)
+        return boundsurf(surf(Ld, L), surf(Ud, U), DefiniteRange, domain)
     else:
         Ld2 = dictSum(l2)
-        Ud2 = Ld2 if sameBounds else dictSum(u2)
-        if sameBounds:
-            s1 = s2 = surf2(Ld2, Ld, L)
-        else:
-            s1, s2 = surf2(Ld2, Ld, L), surf2(Ud2, Ud, U)
-        return boundsurf2(s1, s2, DefiniteRange, domain)
+        Ud2 = dictSum(u2)
+        return boundsurf2(surf2(Ld2, Ld, L), surf2(Ud2, Ud, U), DefiniteRange, domain)
 
 def sum_derivative(r_, r0, INP, dep, point, fixedVarsScheduleID, Vars=None, fixedVars = None, useSparse = 'auto'):
     # TODO: handle involvePrevData
@@ -1201,19 +1174,6 @@ def sum(inp, *args, **kwargs):
         r_dep = r._getDep()
         r._D = lambda *args, **kw: sum_derivative(r, r0, INP, r_dep, *args, **kw)
         r.isCostly = True
-        
-        def expression(*args, **kw):
-            Elems = [elem.expression(**kw) if isinstance(elem, oofun) else str(elem) for elem in INP]
-            r = []
-            for i in range(len(Elems)):
-                r.append(Elems[i] +  (' + ' if i != len(Elems)-1 and Elems[i+1][0] != '-' else ' '))
-            rr = ''.join(r)[:-1]
-            if not np.array_equal(r0, 0.0):
-                str_r0 = str(r0)
-                rr += ' + ' + str_r0 if str_r0[0] != '-' else ' - ' + str_r0[1:]
-            return rr
-        r.expression = expression
-        
         return r
     else: 
         return inp.sum(*args, **kwargs)#np.sum(inp, *args, **kwargs)
@@ -1230,15 +1190,10 @@ def prod(inp, *args, **kwargs):
 def norm(*args, **kwargs):
     if len(kwargs) or len(args) > 1:
         return np.linalg.norm(*args, **kwargs)
-        
     r = sqrt(sum(args[0]**2),  attachConstraints=False)
-
-    if isinstance(r, oofun) and len(kwargs) == 0:
-        assert len(args) == 1 or args[1] == 2, 'FuncDesigner norm() is implemented for n = 2 only'
+    if isinstance(r, oofun) and len(kwargs) == 0 and (len(args) == 1 or args[1] == 2):
         r.engine = 'norm2'
         r._norm_arg = args[0]#used in SOCP
-    else:
-        assert 0, 'FuncDesigner norm() is implemented for n = 2 only'
     return r
  
 __all__ += ['sum', 'prod', 'norm']
@@ -1304,7 +1259,7 @@ def ifThenElse(condition, val1, val2, *args, **kwargs):
     elif isinstance(condition, oofun):
         f = lambda conditionResult, value1Result, value2Result: value1Result if conditionResult else value2Result
         # !!! Don't modify it elseware function will evaluate both expressions despite of condition value 
-        r = oofun(f, [condition, val1, val2], engine = 'ifThenElse')
+        r = oofun(f, [condition, val1, val2])
         r.D = lambda point, *args, **kwargs: (Val1.D(point, *args, **kwargs) if isinstance(Val1, oofun) else {}) if condition(point) else \
         (Val2.D(point, *args, **kwargs) if isinstance(Val2, oofun) else {})
         r._D = lambda point, *args, **kwargs: (Val1._D(point, *args, **kwargs) if isinstance(Val1, oofun) else {}) if condition(point) else \
@@ -1365,7 +1320,7 @@ def max(inp,  *args,  **kwargs):
             r1[r1<tmp] = tmp
             r2[r2<tmp] = tmp
             return np.vstack((r1, r2)), DefiniteRange
-        r = oofun(f, inp, size = 1, _interval_ = interval, engine = 'max')
+        r = oofun(f, inp, size = 1, _interval_ = interval)
         def _D(point, *args, **kwargs):
             ind = np.argmax([(s(point) if isinstance(s, oofun) else s) for s in r.input])
             return r.input[ind]._D(point, *args, **kwargs) if isinstance(r.input[ind], oofun) else {}
@@ -1412,7 +1367,7 @@ def min(inp,  *args,  **kwargs):
                 r2[r2>tmp] = tmp
             return np.vstack((r1, r2)), DefiniteRange
             
-        r = oofun(f, inp, size = 1, _interval_ = interval, engine = 'min')
+        r = oofun(f, inp, size = 1, _interval_ = interval)
         def _D(point, *args, **kwargs):
             ind = np.argmin([(s(point) if isinstance(s, oofun) else s) for s in r.input])
             return r.input[ind]._D(point, *args, **kwargs) if isinstance(r.input[ind], oofun) else {}
@@ -1475,7 +1430,7 @@ def hstack(tup): # overload for oofun[ind]
         return np.max(orders)
     
             
-    r = oofun(f, tup, getOrder = getOrder, engine = 'hstack')
+    r = oofun(f, tup, getOrder = getOrder)
     
     #!!!!!!!!!!!!!!!!! TODO: sparse 
 
@@ -1533,11 +1488,9 @@ def get_inner_coeffs(func, func_d, d, l, u, d_l, d_u, c_l, c_u, pointCase, lineC
         ll, uu = d_u * l + c_u, d_u * u + c_u
     else: # parabola must be below the func
         ll, uu = d_l * l + c_l, d_l * u + c_l
-    
-    ind_infeas = False
+        
     if feasLB != -np.inf:
-        ind_infeas = logical_and(ll<feasLB, uu>feasLB)
-        ll[ind_infeas] = feasLB
+        ll[logical_and(ll<feasLB, uu>feasLB)] = feasLB
     f_l, f_u = func(ll), func(uu)
 
     if pointCase == 'u':
@@ -1552,11 +1505,8 @@ def get_inner_coeffs(func, func_d, d, l, u, d_l, d_u, c_l, c_u, pointCase, lineC
         
     ind_z = np.logical_or(l == u, np.logical_not(np.isfinite(b)))
     P = 1e10
-    #ind_numericaly_unstable = P  < np.abs(a * l) + np.abs(b)
-    ind_numericaly_unstable = np.logical_or(P  < np.abs(a * l), P < np.abs(b))
-    
+    ind_numericaly_unstable = P  < np.abs(a * l) + np.abs(b)
     ind_z = np.logical_or(ind_z, ind_numericaly_unstable)
-    ind_z = np.logical_or(ind_z, ind_infeas)
     a[ind_z] = b[ind_z] = 0.0
     
     c = f_l - (a * l + b) * l
